@@ -4,7 +4,7 @@
 --[[
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (c) 1993-2019 Otfried Cheong
+    Copyright (c) 1993-2020 Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -97,6 +97,7 @@ function MODEL:paction(a1)
       self:warning("Operation '" .. a .. "' is not yet implemented")
     end
   end
+  self:checkType3Font()
 end
 
 local function has_text(page)
@@ -128,11 +129,19 @@ function MODEL:selector(prop, value)
     if not name then return end -- not found
     value = name
   end
+  if value == "<absolute>" then
+    local proptype = prop
+    if prop == "stroke" or prop == "fill" then proptype = "color" end
+    value = self.attributes[prop]
+    if type(value) == "string" then
+      value = self:findOldValue(prop, proptype)
+    end
+  end
   self.attributes[prop] = value
   self.ui:setAttributes(self.doc:sheets(), self.attributes)
   if self:page():hasSelection() then
     self:setAttribute(prop, value)
-    if prop == "textsize" or has_text(self:page()) then
+    if prop == "textsize" or (prop == "stroke" and has_text(self:page())) then
       self:autoRunLatex()
     end
   end
@@ -140,15 +149,21 @@ function MODEL:selector(prop, value)
 end
 
 function MODEL:set_absolute(prop, value)
+  self.attributes[prop] = value
+  self.ui:setAttributes(self.doc:sheets(), self.attributes)
   if self:page():hasSelection() then
     self:setAttribute(prop, value)
   end
+  -- self:print_attributes()
 end
 
 function MODEL:layerAction(a, layer, target)
   local name = a
   local arg = nil
-  if a:sub(-2) == "on" then
+  if a:sub(1,4) == "snap" then
+    name = "snap"
+    arg = a:sub(5)
+  elseif a:sub(-2) == "on" then
     name = a:sub(1,-3)
     arg = true
   elseif a:sub(-3) == "off" then
@@ -188,6 +203,12 @@ function MODEL:absoluteButton(button)
 			     old.r, old.g, old.b)
     if r then
       self:set_absolute(button, { r = r, g = g, b = b });
+    elseif config.platform == "apple" then
+      self:warning("Selecting absolute colors",
+      	           "I have displayed the MacOS color selection dialog.\n\n" ..
+		   "After you have found your color in this dialog, " ..
+		   "you'll need to press the absolute color button again to " ..
+		   "apply the new color to your selection in Ipe.")
     end
   elseif button == "pen" then
     local old = self:findOldValue(button, button)
@@ -256,6 +277,30 @@ function MODEL:action_mark_view(m)
   self:register(t)
 end
 
+function MODEL:mark_views_from_view(m, label)
+  local t = { label=label,
+	      pno=self.pno,
+	      vno=self.vno,
+	      original=self:page():clone(),
+	      undo=revertOriginal,
+	      final=m,
+	    }
+  t.redo = function (t, doc)
+	     for i = t.vno, doc[t.pno]:countViews() do
+	       doc[t.pno]:setMarkedView(i, t.final)
+	     end
+	   end
+  self:register(t)
+end
+
+function MODEL:action_mark_from_view()
+  self:mark_views_from_view(true, "mark some views")
+end
+
+function MODEL:action_unmark_from_view()
+  self:mark_views_from_view(false, "unmark some views")
+end
+
 function MODEL:action_mark_page(m)
   local t = { label="set page mark to " .. tostring(m),
 	      pno=self.pno,
@@ -339,10 +384,15 @@ function MODEL:showLayerBoxPopup(v, layer)
   else
     m:add("lockon", "Lock " .. layer)
   end
-  if p:hasSnapping(layer) then
-    m:add("snapoff", "Disable snapping for " .. layer)
-  else
-    m:add("snapon", "Enable snapping for " .. layer)
+  local sm = p:snapping(layer)
+  if sm ~= "never" then
+    m:add("snapnever", "Disable snapping for " .. layer)
+  end
+  if sm ~= "visible" then
+    m:add("snapvisible", "Snap to " .. layer .. " when visible")
+  end
+  if sm ~= "always" then
+    m:add("snapalways", "Always snap to " .. layer)
   end
   local layers = p:layers()
   if #layers > 1 then
@@ -452,7 +502,7 @@ function MODEL:layeraction_snap(layer, arg)
 	      pno=self.pno,
 	      vno=self.vno,
 	      layer=layer,
-	      original=p:hasSnapping(layer),
+	      original=p:snapping(layer),
 	      snapping=arg,
 	    }
   t.undo = function (t, doc)
@@ -614,6 +664,7 @@ function MODEL:action_ipelet(a)
       else
 	ipelet.run(self, method)
       end
+      self:page():ensurePrimarySelection()
       break
     end
   end
@@ -640,7 +691,7 @@ end
 
 function MODEL:action_auto_latex()
   self.auto_latex = self.ui:actionState("auto_latex")
-  print("Auto is", self.auto_latex)
+  -- print("Auto is", self.auto_latex)
 end
 
 ----------------------------------------------------------------------
@@ -814,6 +865,7 @@ end
 
 ----------------------------------------------------------------------
 
+-- this function is called without arguments on OSX from ipeAlwaysAction
 function action_manual()
   local url = config.docdir .. "/manual.html"
   if ipeui.startBrowser then
@@ -829,12 +881,14 @@ function MODEL:action_manual()
   action_manual()
 end
 
+-- this function is called without arguments on OSX from ipeAlwaysAction
 function action_show_libraries(win)
   local s = "Ipe relies on the following fine pieces of software:\n"
   s = s .. "\n * Pdftex, Xetex, or Luatex"
   s = s .. "\n * " .. _VERSION .. " (" .. math.floor(collectgarbage("count")) .. " kB used)"
   s = s .. "\n * The font rendering library " .. config.freetype_version
   s = s .. "\n * The rendering library Cairo " .. config.cairo_version
+  s = s .. "\n * The clothoid spline library libspiro " .. config.spiro_version
   if config.qt_version then
     s = s .. "\n * The GUI toolkit Qt " .. config.qt_version
   end
@@ -842,7 +896,8 @@ function action_show_libraries(win)
   messageBox(win, "information", "Ipe libraries", s)
 end
 
-function action_show_configuration(win)
+-- this function is called without arguments on OSX from ipeAlwaysAction
+function action_show_configuration(win, dpi)
   local s = ""
   s = s .. " * Lua code: " .. package.path
   s = s .. "\n * Style directories:\n  - " ..
@@ -856,18 +911,24 @@ function action_show_configuration(win)
   s = s .. "\n * Latex program path: " .. config.latexpath
   s = s .. "\n * Latex directory: " .. config.latexdir
   s = s .. "\n * Icons: " .. config.icons
+  s = s .. "\n * Screen geometry: " .. config.screen_geometry[1] .. " x " .. config.screen_geometry[2]
+  if config.platform == "win" then
+    s = s .. "\n * Monitor resolution: " .. dpi .. " dpi"
+  end
   s = s .. "\n * External editor: " .. (prefs.external_editor or "none")
   messageBox(win, "information", "Ipe configuration", s)
 end
 
 function MODEL:action_show_configuration()
-  action_show_configuration(self.ui:win())
+  local cs, dpi = self.ui:canvasSize()
+  action_show_configuration(self.ui:win(), dpi)
 end
 
 function MODEL:action_show_libraries()
   action_show_libraries(self.ui:win())
 end
 
+-- this function is called without arguments on OSX from ipeAlwaysAction
 function action_about_ipelets(win)
   local s = ""
   for i,v in ipairs(ipelets) do
@@ -1493,10 +1554,53 @@ function MODEL:action_delete_view()
   self.ui:explain(t.label)
 end
 
+function attribute_map(map)
+  local s = ""
+  for i,mapping in ipairs(map) do
+    s = s .. mapping.kind .. ":" .. mapping.map .. "=" .. mapping.to .. ";\n"
+  end
+  return s
+end
+
+function parse_attribute_map(s)
+  local kinds = { "pen", "symbolsize", "arrowsize", "opacity", "color", "dashstyle", "symbol" }
+  local map = {}
+  for kind, from, to in string.gmatch(s, "(%l+):([^=]+)=([^;]+);") do
+    if indexOf(kind, kinds) then  -- silently ignore unknown kinds
+      map[#map+1] = { kind=kind, map=from, to=to }
+    end
+  end
+  return map
+end
+
+function layer_matrices(tfm)
+  local s = ""
+  for l, m in pairs(tfm) do
+    s = s .. l .. "=" .. tostring(m) .. ";\n"
+  end
+  return s
+end
+
+function parse_layer_matrices(s)
+  local map = {}
+  for l, ms in string.gmatch(s, "([^%s=]+)=%s*%[([%d%s.e-]+)%]%s*;") do
+    local m = {}
+    for k in string.gmatch(ms, "%S+") do
+      m[#m+1] = tonumber(k)
+    end
+    if #m == 6 then
+      map[l] = ipe.Matrix(m)
+    end
+  end
+  return map
+end
+
 function MODEL:action_edit_view()
   local p = self:page()
   local effect0 = p:effect(self.vno)
   local name0 = p:viewName(self.vno)
+  local map0 = p:viewMap(self.vno)
+  local tfm0 = p:layerMatrices(self.vno)
   local list = self.doc:sheets():allNames("effect")
   table.insert(list, 1, "normal")
   local d = ipeui.Dialog(self.ui:win(), "Edit view")
@@ -1504,31 +1608,49 @@ function MODEL:action_edit_view()
   d:add("label", "label", { label=label }, 1, 1, 2, 1)
   d:add("name-label", "label", { label="Name" }, 0, 1)
   d:add("name", "input", {}, -1, 2)
+  d:add("map-label", "label", { label="Attribute map" }, 0, 1)
+  d:add("map", "text", {}, -1, 2)
+  d:add("tfm-label", "label", { label="Transformations" }, 0, 1)
+  d:add("tfm", "text", {}, -1, 2)
   d:add("effect-label", "label", { label="Effect" }, 0, 1)
   d:add("combo", "combo", list, -1, 2)
   d:addButton("ok", "&Ok", "accept")
   d:addButton("cancel", "&Cancel", "reject")
   d:set("name", name0)
+  d:set("map", attribute_map(map0))
+  d:set("tfm", layer_matrices(tfm0))
   d:set("combo", indexOf(effect0, list))
   if not d:execute() then return end
   local name1 = d:get("name")
   local effect1 = list[d:get("combo")]
+  local map1 = parse_attribute_map(d:get("map"))
+  local tfm1 = parse_layer_matrices(d:get("tfm"))
   local t = { label="modify view " .. self.vno,
 	      pno = self.pno,
 	      vno = self.vno,
 	      name0 = name0,
 	      effect0 = effect0,
+	      map0 = map0,
+	      tfm0 = tfm0,
 	      name1 = name1,
 	      effect1 = effect1,
+	      map1 = map1,
+	      tfm1 = tfm1,
 	    }
   t.undo = function (t, doc)
-	     p:setEffect(t.vno, t.effect0)
-	     p:setViewName(t.vno, t.name0)
-	   end
+    local p = doc[t.pno]
+    p:setEffect(t.vno, t.effect0)
+    p:setViewName(t.vno, t.name0)
+    p:setViewMap(t.vno, t.map0)
+    p:setLayerMatrices(t.vno, t.tfm0)
+  end
   t.redo = function (t, doc)
-	     p:setEffect(t.vno, t.effect1)
-	     p:setViewName(t.vno, t.name1)
-	   end
+    local p = doc[t.pno]
+    p:setEffect(t.vno, t.effect1)
+    p:setViewName(t.vno, t.name1)
+    p:setViewMap(t.vno, t.map1)
+    p:setLayerMatrices(t.vno, t.tfm1)
+  end
   self:register(t)
 end
 
@@ -1564,8 +1686,7 @@ end
 
 function MODEL:action_cut_page()
   if #self.doc == 1 then
-    self:warning("Cannot cut page",
-		 "This is the only page of the document.")
+    self:warning("Cannot cut page", "This is the only page of the document.")
     return
   end
   local t = { label="cut page " .. self.pno,
@@ -1635,6 +1756,7 @@ function MODEL:action_paste_page()
   self:register(t)
   self:nextPage(1)
   self:setPage()
+  self:autoRunLatex()
 end
 
 function MODEL:action_delete_page()
@@ -1764,6 +1886,8 @@ function MODEL:action_edit_notes()
   self:register(t)
 end
 
+----------------------------------------------------------------------
+
 -- rearrange pages of document
 -- new order is given by arr
 -- 0 entries in arr are taken from newpages
@@ -1792,14 +1916,25 @@ function arrange_pages(doc, arr, newpages)
   return unused
 end
 
+function mark_pages(doc, marks)
+  local originalMarks = {}
+  for i = 1,#doc do
+    local p = doc[i]
+    originalMarks[i] = p:marked()
+    p:setMarked(marks[i])
+  end
+  return originalMarks
+end
+
 function MODEL:action_page_sorter()
-  local arr = self.ui:pageSorter(self.doc)
+  local arr, marks = self.ui:pageSorter(self.doc)
   if not arr then return end -- canceled
   if #arr == 0 then
     self:warning("You cannot delete all pages of the document")
     return
   end
-  printTable(arr)
+  -- printTable(arr)
+  -- printTable(marks)
   -- compute reverse 'permutation'
   local rev = {}
   for i = 1,#self.doc do rev[i] = 0 end
@@ -1809,11 +1944,15 @@ function MODEL:action_page_sorter()
 	      vno = 1,
 	      arr = arr,
 	      rev = rev,
+	      marks = marks,
+	      originalMarks = marks
 	    }
   t.undo = function (t, doc)
 	     arrange_pages(doc, t.rev, t.deleted)
+	     mark_pages(doc, t.originalMarks)
 	   end
   t.redo = function (t, doc)
+	     t.originalMarks = mark_pages(doc, t.marks)
 	     t.deleted = arrange_pages(doc, t.arr, {})
 	   end
   self.pno = 1
@@ -1821,6 +1960,68 @@ function MODEL:action_page_sorter()
   self:register(t)
 end
 
+----------------------------------------------------------------------
+
+-- rearrange views of a page
+-- new order is given by arr
+function arrange_views(p, arr, marks)
+  -- collect view information from the page
+  local views = {}
+  local layers = p:layers()
+  for i = 1,p:countViews() do
+    local visible = {}
+    for _,l in ipairs(layers) do
+      visible[l] = p:visible(i, l)
+    end
+    local v = {
+      name=p:viewName(i),
+      marked=marks[i],
+      active=p:active(i),
+      effect=p:effect(i),
+      visible=visible,
+    }
+    views[#views+1] = v
+  end
+  p:clearViews()
+  -- insert views in new order
+  for i = 1,#arr do
+    local v = views[arr[i]]
+    p:insertView(i, v.active)
+    p:setEffect(i, v.effect)
+    p:setMarkedView(i, v.marked)
+    p:setViewName(i, v.name)
+    for _,l in ipairs(layers) do
+      p:setVisible(i, l, v.visible[l])
+    end
+  end
+end
+
+function MODEL:action_view_sorter()
+  local arr, marks = self.ui:pageSorter(self.doc, self.pno)
+  if not arr then return end -- canceled
+  if #arr == 0 then
+    self:warning("You cannot delete all views of the page")
+    return
+  end
+  -- printTable(arr)
+  -- printTable(marks)
+  local t = { label="rearrangement of views",
+	      pno=self.pno,
+	      vno=1,
+	      arr=arr,
+	      marks=marks,
+	      original=self:page():clone(),
+	      undo=revertOriginal,
+	    }
+  t.redo = function (t, doc)
+	     arrange_views(doc[t.pno], t.arr, t.marks)
+	   end
+  self.vno = 1
+  self:register(t)
+end
+
+----------------------------------------------------------------------
+
 function MODEL:action_toggle_notes()
   self.ui:showTool("notes", self.ui:actionState("toggle_notes"))
   self.ui:setNotes(self:page():notes())
@@ -2235,6 +2436,65 @@ function MODEL:action_paste()
 	   end
   p:deselectAll()
   self:register(t)
+  self:autoRunLatex()
+end
+
+function MODEL:action_paste_with_layer()
+  local data = self.ui:clipboard(false)
+  if not data then
+    self:warning("Nothing to paste")
+    return
+  end
+  if type(data) ~= "string" or data:sub(1,13) ~= "<ipeselection" then
+    self:warning("No Ipe selection to paste")
+    return
+  end
+  local elements, layers = ipe.Object(data)
+  if not elements then
+    self:warning("Could not parse Ipe selection on clipboard")
+    return
+  end
+  local p = self:page()
+  local active = p:active(self.vno)
+  local pLayers = p:layers()
+  local impossible = {}
+  for i,l in ipairs(layers) do
+    if l == "" or indexOf(l, pLayers) == nil or p:isLocked(l) then
+      if indexOf(l, impossible) == nil then
+	impossible[#impossible+1] = l
+      end
+      layers[i] = active  -- place in active layer instead
+    end
+  end
+  if #impossible > 0 then
+    local impossibleText = ""
+    for _,l in ipairs(impossible) do
+      impossibleText = impossibleText .. l .. ", "
+    end
+    self:warning("Impossible to paste to specified layer",
+		 "Some of the layers you are trying to paste to either do " ..
+		   "not exist on this page, or are locked.\n\n" ..
+		   "I have used the active layer instead.\n\n" ..
+		   "The following layer names could not be used:\n\n" .. impossibleText)
+  end
+  local t = { label="paste objects with layer",
+	      pno = self.pno,
+	      vno = self.vno,
+	      elements = elements,
+	      layers = layers,
+	      original = p:clone(),
+	      undo = revertOriginal,
+	    }
+  t.redo = function (t, doc)
+	     local p = doc[t.pno]
+	     for i,obj in ipairs(t.elements) do
+	       p:insert(nil, obj, 2, t.layers[i])
+	     end
+	     p:ensurePrimarySelection()
+	   end
+  p:deselectAll()
+  self:register(t)
+  self:autoRunLatex()
 end
 
 ----------------------------------------------------------------------
@@ -2294,6 +2554,7 @@ function MODEL:action_update_style_sheets()
 	   end
   self:register(t)
   self:action_check_style()
+  self:autoRunLatex()
 end
 
 local function sheets_namelist(list)
@@ -2511,6 +2772,7 @@ function MODEL:action_document_properties()
   t.undo = function (t, doc) doc:setProperties(t.original) end
   t.redo = function (t, doc) doc:setProperties(t.final) end
   self:register(t)
+  self:autoRunLatex()
 end
 
 ----------------------------------------------------------------------
