@@ -4,7 +4,7 @@
 --[[
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (c) 1993-2019 Otfried Cheong
+    Copyright (c) 1993-2020 Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -206,6 +206,7 @@ else
 end
 filter_png = { "Images (*.png)", "*.png" }
 filter_eps = { "Postscript files (*.eps)", "*.eps" }
+filter_svg = { "SVG files (*.svg)", "*.svg" }
 
 ----------------------------------------------------------------------
 -- This function is called to launch a file on MacOS X
@@ -446,7 +447,7 @@ local style_sheets = {}
 while i <= #argv do
   if argv[i] == "-sheet" then
     if i == #argv then usage() return end
-    style_sheets[#style_sheets + 1] = ipe.realPath(argv[i+1])
+    style_sheets[#style_sheets + 1] = argv[i+1]
     i = i + 2
   else
     if i ~= #argv then usage() return end
