@@ -4,7 +4,7 @@
 --[[
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (c) 1993-2019 Otfried Cheong
+    Copyright (c) 1993-2020 Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -41,8 +41,19 @@ prefs.styles = { "basic" }
 -- Possible values are "default", "pdftex", "xetex", and "luatex"
 prefs.tex_engine = "default"
 
+-- What to display on the "View" and "Page" button
+-- Standard:
+prefs.view_button_prefix = "View "
+prefs.page_button_prefix = "Page "
+-- Concise:
+-- prefs.view_button_prefix = "V "
+-- prefs.page_button_prefix = "P "
+-- Expert:
+-- prefs.view_button_prefix = ""
+-- prefs.page_button_prefix = ""
+
 -- URL for a cloud service that provides Latex compilation
-prefs.latex_service_url = "https://latexonline.cc/compile"
+prefs.latex_service_url = "https://latexonline.cc"
 
 -- Interval for autosaving in seconds
 -- set to nil to disable autosaving
@@ -52,6 +63,9 @@ prefs.autosave_interval = 600 -- 10 minutes
 -- can contain '%s' for the filename of the current file
 -- can use 'home' for the user's home directory
 -- prefs.autosave_filename = home .. "/autosave.ipe"
+-- On Windows, config.ipedrive gives you the root of the drive
+-- where Ipe is installed.
+-- prefs.autosave_filename = config.ipedrive .. "/autosave.ipe"
 if config.platform == "win" then
   prefs.autosave_filename = config.documents .. "\\%s.autosave"
 else
@@ -82,9 +96,11 @@ else
     prefs.external_editor = "open -W -n -e %s"
   else
     -- prefs.external_editor = "xed %s"
-    -- prefs.external_editor = "emacsclient %s"
-    prefs.external_editor = nil
   end
+  -- For Emacs users:
+  -- prefs.external_editor = "emacsclient %s"
+  -- if you use Visual studio, you make like the following:
+  -- prefs.external_editor = "code --wait %s"
 end
 
 -- The name of the temporary textfile where Ipe will store text to be
@@ -118,8 +134,16 @@ prefs.tools_visible = {
   layers = true
 }
 
--- initial ordering of the toolbars (windows only)
-prefs.toolbar_order = { "snap", "grid", "angle", "edit", "mode" }
+-- on OSX, specify a minimum width for the properties and layer panels (in pixels)
+-- to ensure that the width of the canvas does not change while you step through your document
+-- if you change "view_button_prefix" and "page_button_prefix", you can make this smaller
+prefs.osx_properties_width = 200.0
+
+-- on Windows, initial ordering of the toolbars
+prefs.win_toolbar_order = { "snap", "grid", "angle", "edit", "mode" }
+
+-- on Windows, keep edit toolbar minimal?
+prefs.win_mini_edit = false
 
 -- Should LaTeX be run automatically every time text has changed?
 prefs.auto_run_latex = true
@@ -132,10 +156,9 @@ prefs.editor_closes_dialog = nil
 
 -- If the user interface is too small, scale it here (Windows and Qt only)
 -- 100 is the original size, 150 is 50% larger
--- On Windows 10, the system dpi is used automatically if left at 100.
-prefs.ui_scale = 100
--- The following influences the size of the toolbar buttons
-prefs.toolbar_scale = 100
+prefs.ui_scaling = 100
+-- The following scales the size of the toolbar buttons
+prefs.toolbar_scaling = 100
 
 -- Minimum size of dialog window for creating/editing text objects
 prefs.editor_size = { 0, 0 }
@@ -144,7 +167,7 @@ prefs.editor_size = { 0, 0 }
 prefs.width_notes_bookmarks = 200
 
 -- Size of main window at startup
-prefs.window_size = { 1080, 600 }
+prefs.window_size = { 1160, 720 }
 
 -- Size of Latex error message window
 prefs.latexlog_size = { 800, 600 }
@@ -155,6 +178,16 @@ prefs.page_sorter_size = { 960, 600 }
 -- Width of page thumbnails (height is computed automatically)
 prefs.thumbnail_width = 200
 
+-- The laser pointer
+prefs.laser_pointer = {
+  color = { r = 1.0, g = 0.0, b = 0.0 }, -- red
+  pen = 6.0,
+  radius = 7.0,
+}
+
+-- tension of cardinal splines
+prefs.spline_tension = 0.5
+
 -- Canvas customization:
 prefs.canvas_style = {
   paper_color = { r = 1.0, g = 1.0, b = 1.0 },  -- white
@@ -182,6 +215,7 @@ prefs.snap = { vertex = false,
 	       intersection = false,
 	       grid = true,
 	       angle = false,
+	       custom = false,
 	       autoangle = false }
 
 -- UI settings when Ipe starts
@@ -208,6 +242,7 @@ prefs.initial_attributes = {
   textsize = "normal",
   textstyle = "normal",
   transformabletext = false,
+  splinetype = "bspline",
   horizontalalignment = "left",
   verticalalignment = "baseline",
   pinned = "none",
@@ -350,11 +385,18 @@ prefs.ink_spline = false
 
 -- Auto-exporting when document is being saved
 -- If this is set, then every time you save in xml format, say to
--- mydoc.ipe, the same document is EXPORTED to mydoc.pdf.
--- Careful: if mydoc.pdf exists, it is silently overwritten.
--- mydoc.pdf cannot be opened by Ipe (it is an exported copy).
+-- mydoc.ipe, the same document is EXPORTED to mydoc.format,
+-- for each of the enabled formats.
+-- Careful: if the file exists, it is silently overwritten.
+-- The exported files cannot be opened by Ipe (they are exported copies).
+-- Export to eps, png, svg use the first view of the first page only.
 
-prefs.auto_export_to_pdf = false
+-- possible formats are "pdf", "eps", "svg", "png"
+-- e.g. prefs.auto_export = { "pdf", "eps" }
+prefs.auto_export = { }
+
+-- the pixels per inch resolution to use for PNG export
+prefs.auto_export_resolution = 100
 
 ----------------------------------------------------------------------
 
