@@ -1,7 +1,10 @@
+----------------------------------------------------------------------
+-- tools.lua
+----------------------------------------------------------------------
 --[[
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (c) 1993-2019 Otfried Cheong
+    Copyright (c) 1993-2020 Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -25,10 +28,6 @@
 
 --]]
 
-----------------------------------------------------------------------
--- tools.lua
-----------------------------------------------------------------------
-
 function externalEditor(d, field)
   local text = d:get(field)
   local fname = os.tmpname()
@@ -104,6 +103,13 @@ function LINESTOOL:new(model, mode)
   setmetatable(tool, LINESTOOL)
   tool.model = model
   tool.mode = mode
+  tool.splinetype = "spline"
+  if model.attributes.splinetype == "cardinal" then
+    tool.splinetype = "cardinal"
+  elseif model.attributes.splinetype == "spiro" then
+    tool.splinetype = "spiro"
+  end
+  tool.tension = model.attributes.tension
   local v = model.ui:pos()
   tool.v = { v, v }
   model.ui:setAutoOrigin(v)
@@ -178,7 +184,8 @@ function LINESTOOL:compute()
       local j = i
       while j <= #self.v and self.t[j] == SPLINE do j = j + 1 end
       if j > #self.v then j = #self.v end
-      seg = { type="spline" }
+      seg = { type = self.splinetype }
+      if self.splinetype == "cardinal" then seg.tension = prefs.spline_tension end
       for k = i-1,j do seg[#seg+1] = self.v[k] end
       i = j + 1
     elseif self.t[i] == ARC then
@@ -292,6 +299,7 @@ function LINESTOOL:key(text, modifiers)
     text == shortcuts_linestool.set_axis then
     -- set axis
     self.model.snap.with_axes = true
+    self.model.ui:setActionState("show_axes", self.model.snap.with_axes)
     self.model.snap.snapangle = true
     self.model.snap.origin = self.v[#self.v - 1]
     self.model.snap.orientation = self:compute_orientation()
@@ -925,7 +933,7 @@ function MODEL:createText(mode, pos, width, pinned)
   d:addButton("cancel", "&Cancel", "reject")
   d:setStretch("row", 2, 1)
   d:setStretch("column", 2, 1)
-  d:set("ignore-escape", true)
+  d:set("ignore-escape", "text", "")
   local style = indexOf(styleatt, styles)
   if not style then style = indexOf("normal", styles) end
   local size = indexOf(self.attributes.textsize, sizes)
@@ -1021,10 +1029,10 @@ end
 
 ----------------------------------------------------------------------
 
-local function collect_edges(p, box)
+local function collect_edges(p, box, view)
   local edges = {}
   for i, obj, sel, layer in p:objects() do
-    if obj:type() == "path" then
+    if obj:type() == "path" and (view == nil or p:visible(view, i)) then
       local shape = obj:shape()
       local m = obj:matrix()
       if #shape == 1 and shape[1].type == "curve"
@@ -1075,16 +1083,18 @@ local function findClosestVertex(model)
   end
 end
 
-function GRAPHTOOL:new(model)
+function GRAPHTOOL:new(model, moveInvisibleEdges)
   if not findClosestVertex(model) then return end
   local p = model:page()
   self.prim = p:primarySelection()
+  local view = model.vno
+  if moveInvisibleEdges then view = nil end
   local tool = {}
   setmetatable(tool, GRAPHTOOL)
   tool.model = model
   tool.orig = model.ui:pos()
   tool.box = p:bbox(self.prim)
-  tool.edges = collect_edges(p, tool.box)
+  tool.edges = collect_edges(p, tool.box, view)
   model.ui:shapeTool(tool)
   tool.setColor(1.0, 0.0, 0.0)
   return tool
@@ -1178,25 +1188,43 @@ function CHANGEWIDTHTOOL:new(model, prim, obj)
   tool.obj = obj
   tool.pos = obj:matrix() * obj:position()
   tool.wid = obj:get("width")
+  tool.align = obj:get("horizontalalignment")
+  if tool.align == "right" then
+    tool.posfactor = -1
+    tool.dir = -1
+  elseif tool.align == "hcenter" then
+    tool.posfactor = -0.5
+    tool.dir = 1
+  else
+    tool.posfactor = 0
+    tool.dir = 1
+  end
   model.ui:shapeTool(tool)
   tool.setColor(1.0, 0, 1.0)
-  tool.setShape( { boxshape(tool.pos, tool.pos + V(tool.wid, -20)) } )
+  local pos = tool.pos + V(tool.posfactor * tool.wid, 0)
+  tool.setShape( { boxshape(pos, pos + V(tool.wid, -20)) } )
   model.ui:update(false) -- update tool
   return tool
 end
 
 function CHANGEWIDTHTOOL:compute()
   local w = self.model.ui:pos()
-  self.nwid = self.wid + (w.x - self.v.x)
-  self.shape = boxshape(self.pos, self.pos + V(self.nwid, -20))
+  self.nwid = self.wid + self.dir * (w.x - self.v.x)
+  local pos = self.pos + V(self.posfactor * self.nwid, 0)
+  self.shape = boxshape(pos, pos + V(self.nwid, -20))
 end
 
 function CHANGEWIDTHTOOL:mouseButton(button, modifiers, press)
   if press then
     if not self.v then self.v = self.model.ui:pos() end
+    if self.align == "hcenter" and self.v.x < self.pos.x then self.dir = -1 end
   else
     self:compute()
     self.model.ui:finishTool()
+    if self.nwid <= 0 then
+      self.model:warning("The width of a text object should be positive.")
+      return
+    end
     self.model:setAttributeOfPrimary(self.prim, "width", self.nwid)
     self.model:autoRunLatex()
   end
@@ -1250,7 +1278,7 @@ function MODEL:startModeTool(modifiers)
   elseif self.mode == "shredder" then
     self:shredObject()
   elseif self.mode == "graph" then
-    GRAPHTOOL:new(self)
+    GRAPHTOOL:new(self, modifiers.shift)
   elseif self.mode:sub(1,10) == "rectangles" or self.mode == "parallelogram" then
     BOXTOOL:new(self, modifiers.shift, self.mode)
   elseif self.mode == "splinegons" then
@@ -1270,6 +1298,8 @@ function MODEL:startModeTool(modifiers)
     self:createText(self.mode, self.ui:pos())
   elseif self.mode == "paragraph" then
     PARAGRAPHTOOL:new(self)
+  elseif self.mode == "laser" then
+    LASERTOOL:new(self)
   else
     print("start mode tool:", self.mode)
   end
@@ -1326,6 +1356,11 @@ end
 function apply_text_edit(d, data, run_latex)
   -- refuse to do anything with empty text
   if string.match(d:get("text"), "^%s*$") then return end
+  if data.obj:text() == d:get("text") and
+     (not data.size or data.obj:get("textsize") == data.sizes[d:get("size")]) and
+     (not data.style or data.obj:get(data.stylekind) == data.styles[d:get("style")]) then
+     return -- hasn't changed since last time
+  end
   local model = data.model
   local final = data.obj:clone()
   final:setText(d:get("text"))
@@ -1373,7 +1408,7 @@ function MODEL:action_edit_text(prim, obj)
   d:setStretch("row", 2, 1)
   d:setStretch("column", 2, 1)
   d:set("text", obj:text())
-  d:set("ignore-escape", true)
+  d:set("ignore-escape", "text", obj:text())
   data.style = indexOf(obj:get(stylekind), data.styles)
   if data.style then
     d:add("style", "combo", data.styles, 1, 3)
@@ -1443,7 +1478,7 @@ function MODEL:action_edit_group_text(prim, obj)
   d:setStretch("row", 2, 1)
   d:setStretch("column", 2, 1)
   d:set("text", tobj:text())
-  d:set("ignore-escape", true)
+  d:set("ignore-escape", "text", tobj:text())
   if prefs.auto_external_editor then
     externalEditor(d, "text")
   end
@@ -1485,15 +1520,21 @@ local function start_group_edit(t, doc)
   local elements = g:elements()
   local matrix = g:matrix()
   local layer = "EDIT-GROUP"
-  if g:get("decoration") ~= "normal" then
-    layer = layer .. "<" .. g:get("decoration"):sub(12) .. ">"
-  end
-  if layerOfPrim ~= activeLayer then layer = layer .. "-" .. layerOfPrim end
+  -- make sure new layer name is unique
   while indexOf(layer, layers) do
-    layer = layer:sub(1,10) .. "*" .. layer:sub(11)
+    layer = layer .. "*"
+  end
+  local data = "active=" .. activeLayer .. ";primary=" .. layerOfPrim .. ";"
+  if g:get("decoration") ~= "normal" then
+    data = data .. "decoration=" .. g:get("decoration"):sub(12) .. ";"
+  end
+  data = data .. "locked="
+  for _,l in ipairs(layers) do
+    if p:isLocked(l) then data = data .. l .. "," end
   end
   p:remove(t.primary)
   p:addLayer(layer)
+  p:setLayerData(layer, data)
   p:moveLayer(layer, activeIndex + 1)
   for _,obj in ipairs(elements) do
     p:insert(nil, obj, nil, layer)
@@ -1507,29 +1548,8 @@ end
 
 local function end_group_edit(t, doc)
   local p = doc[t.pno]
-  local layers = p:layers()
   local activeLayer = p:active(t.vno)
-  local activeIndex = indexOf(activeLayer, layers)
-  local origActive
-  if activeIndex > 1 then
-    origActive = layers[activeIndex - 1]
-  else
-    origActive = layers[2]
-  end
-  local origGroup = nil
-  local deco, i = string.match(activeLayer, "^EDIT%-GROUP%**<([^>]+)>()")
-  if not deco then
-    deco = "normal"
-    local og = string.match(activeLayer, "^EDIT%-GROUP%**%-(.*)$")
-    if og then origGroup = og end
-  else
-    deco = "decoration/" .. deco
-    local og = string.match(activeLayer, "^EDIT%-GROUP%**<[^>]+>%-(.*)$")
-    if og then origGroup = og end
-  end
-  if not origGroup or not indexOf(origGroup, layers) then
-    origGroup = origActive
-  end
+  local data = p:layerData(activeLayer)
   local elements = {}
   for i, obj, sel, layer in p:objects() do
     if layer == activeLayer then
@@ -1541,16 +1561,37 @@ local function end_group_edit(t, doc)
       p:remove(i)
     end
   end
-  local group = ipe.Group(elements)
-  group:set("decoration", deco)
-  p:setLocked(origActive, false)
-  p:setLocked(origGroup, false)
-  p:setActive(t.vno, origActive)
   p:removeLayer(activeLayer)
-  p:insert(nil, group, 1, origGroup)
-  if not string.match(origActive, "^EDIT") then
-    for _,l in ipairs(p:layers()) do p:setLocked(l, false) end
+  local group = ipe.Group(elements)
+  for _,l in ipairs(p:layers()) do p:setLocked(l, false) end
+  print("End group edit: ", data)
+  local newActive = nil
+  local layerOfGroup = nil
+  for w in string.gmatch(data, "%w+=[^;]*") do
+    if w:sub(1, 11) == "decoration=" then
+      group:set("decoration", "decoration/" .. w:sub(12))
+    end
+    if w:sub(1, 7) == "active=" then
+      newActive = w:sub(8)
+    end
+    if w:sub(1, 8) == "primary=" then
+      layerOfGroup = w:sub(9)
+    end
+    if w:sub(1, 7) == "locked=" then
+      locked = w:sub(8)
+      for lock in string.gmatch(locked, "[^,]+") do
+	p:setLocked(lock, true)
+      end
+    end
   end
+  -- to be safe, if the page was tampered with
+  if not newActive then
+    newActive = p:layers()[1] -- just use first layer of the page
+  end
+  if p:isLocked(newActive) then p:setLocked(newActive, false) end
+  if not layerOfGroup then layerOfGroup = newActive end
+  p:setActive(t.vno, newActive)
+  p:insert(nil, group, 1, layerOfGroup)
 end
 
 function MODEL:saction_edit_group()
@@ -1688,3 +1729,42 @@ function MODEL:action_paste_at_cursor()
 end
 
 ----------------------------------------------------------------------
+
+LASERTOOL = {}
+LASERTOOL.__index = LASERTOOL
+
+function LASERTOOL:new(model)
+  local tool = {}
+  _G.setmetatable(tool, LASERTOOL)
+  tool.model = model
+  tool.pos = model.ui:pos()
+  model.ui:shapeTool(tool)
+  local p = prefs.laser_pointer.color
+  tool.setColor(p.r, p.g, p.b)
+  tool:setPosShape()
+  return tool
+end
+
+function LASERTOOL:setPosShape()
+  local radius = prefs.laser_pointer.radius
+  self.pos = self.model.ui:pos()
+  local shape = { type="ellipse"; ipe.Matrix(radius, 0, 0, radius, self.pos.x, self.pos.y) }
+  self.setShape( { shape }, 0, prefs.laser_pointer.pen)
+  self.model.ui:update(false) -- update tool
+end
+
+function LASERTOOL:mouseButton(button, modifiers, press)
+  if not press then
+    self.model.ui:finishTool()
+  end
+end
+
+function LASERTOOL:mouseMove()
+  self:setPosShape()
+end
+
+function LASERTOOL:key(text, modifiers)
+  return false
+end
+
+----------------------------------------------------------------------
